package ru.amse.javadependencies.zhukova.analyser;

import ru.amse.javadependencies.zhukova.data.IClass;
import ru.amse.javadependencies.zhukova.data.IPackage;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.DependencyGraph;

import java.util.Enumeration;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.io.*;


import org.objectweb.asm.*;

/**
 * @author Anna R. Zhukova
 */

public class DependenciesAnalyser implements IDependenciesAnalyser {
    private final DependencyGraph<IClass> myClassDependencyGraph = new DependencyGraph<IClass>();
    private final DependencyGraph<IPackage> myPackageDependencyGraph = new DependencyGraph<IPackage>();
    private final File myPath;

    public DependenciesAnalyser(File path) {
        myPath = path;
    }

    public static void main(String[] args) {
        DependenciesAnalyser da = new DependenciesAnalyser(new File("C:\\Users\\sony\\Desktop\\Новая папка (2)\\classes"));
        //DependenciesAnalyser da = new DependenciesAnalyser("C:\\Users\\sony\\Desktop\\amse\\SampleProject\\classes\\production\\SampleProject.jar");
        try {
            da.analyse();
            System.out.println(da.myClassDependencyGraph);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void analyseJar(File path) throws IOException {
        ZipFile f = new ZipFile(path);
        Enumeration< ? extends ZipEntry> en = f.entries();
        while (en.hasMoreElements()) {
            ZipEntry e = en.nextElement();
            String name = e.getName();
            if (name.endsWith(".class")) {
                SpecialClassVisitor scv = new SpecialClassVisitor(myClassDependencyGraph, myPackageDependencyGraph);
                new ClassReader(f.getInputStream(e)).accept(scv, 0);
            }
        }
    }

    public void analyse() throws IOException {
        analyse(myPath);
    }

    private void analyse(File dir) throws IOException {
        String path = dir.getAbsolutePath();
        if (path.endsWith(".jar") || path.endsWith(".zip")) {
            analyseJar(dir);
        } else {
            if (dir.isDirectory()) {
                File[] files = dir.listFiles();
                if (files != null) {
                    for (File f : files) {
                        analyse(f);
                    }
                }
            } else if (path.endsWith(".class")) {
                analyseClassFile(dir);
            }
        }
    }

    private void analyseClassFile(File dir) throws IOException {
        SpecialClassVisitor scv = new SpecialClassVisitor(myClassDependencyGraph, myPackageDependencyGraph);
        InputStream is = new FileInputStream(dir.getAbsolutePath());
        (new ClassReader(is)).accept(scv, 0);
    }

    public static String replaceSlashesWithDots(String s) {
        return s.replace('\\', '.').replace('/', '.');
    }

    public DependencyGraph<IPackage> getPackageDependencyGraph() {
        return myPackageDependencyGraph;
    }

    public DependencyGraph<IClass> getClassDependencyGraph() {
        return myClassDependencyGraph;
    }
}
