package ru.amse.javadependencies.zhukova.analyser;

import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.Label;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.DependencyGraph;
import ru.amse.javadependencies.zhukova.data.IClass;
import ru.amse.javadependencies.zhukova.data.IPackage;

import static ru.amse.javadependencies.zhukova.analyser.StaticAnaliseMethods.*;

/**
 * @author Anna R. Zhukova
 */
class SpecialMethodVisitor implements MethodVisitor {
    private final IClass myClass;
    private final String myMethodName;
    private final DependencyGraph<IClass> myClassDependencyGraph;
    private final DependencyGraph<IPackage> myPackageDependencyGraph;

    public SpecialMethodVisitor(IClass clazz, String methodName, DependencyGraph<IClass> classDependencyGraph, DependencyGraph<IPackage> packageDependencyGraph) {
        super();
        myClass = clazz;
        myClassDependencyGraph = classDependencyGraph;
        myPackageDependencyGraph = packageDependencyGraph;
        myMethodName = methodName;
    }

    public AnnotationVisitor visitAnnotationDefault() {
        return null;
    }

    public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
        return null;
    }

    public AnnotationVisitor visitParameterAnnotation(int i, String string, boolean b) {
        return null;
    }

    public void visitAttribute(Attribute attr) {
    }

    public void visitCode() {
    }

    public void visitFrame(int i, int i1, Object[] objects, int i2, Object[] objects1) {
    }

    public void visitInsn(int i) {
    }

    public void visitIntInsn(int i, int i1) {
    }

    public void visitVarInsn(int i, int i1) {
    }

    public void visitJumpInsn(int i, Label label) {
    }

    public void visitLabel(Label label) {
    }

    public void visitIincInsn(int i, int i1) {
    }

    public void visitTableSwitchInsn(int i, int i1, Label label, Label[] labels) {
    }

    public void visitLookupSwitchInsn(Label label, int[] ints, Label[] labels) {
    }

    public void visitLineNumber(int i, Label label) {
    }

    public void visitMaxs(int i, int i1) {
    }

    public void visitLocalVariable(String string, String string1, String string2, Label label, Label label1, int i) {
        //System.out.println(myClass.getName() + " : " + myMethodName + " : " + string + " " + string1 + " " + string2 + " " + label + " " + label1 + " " + i);
    }

    public void visitTypeInsn(int i, String string) {
        addElementDependencies(myClassDependencyGraph, myPackageDependencyGraph, myClass, string, myMethodName, null, null);
    }

    // обращения к полям
    public void visitFieldInsn(int i, String string, String string1, String string2) {
        //System.out.println(myClass.getName() + " :: " + myMethodName + ":  " + string + " " + string1 + " " + string2);
        if (!"<init>".equals(string1)) {
            addElementDependencies(myClassDependencyGraph, myPackageDependencyGraph, myClass, string, myMethodName, string1, string2);
        }
    }

    // вызываемые методы
    public void visitMethodInsn(int i, String string, String string1, String string2) {
        if (!"<init>".equals(string1)) {
            addElementDependencies(myClassDependencyGraph, myPackageDependencyGraph, myClass, string, myMethodName, string1, string2);
        }
        //System.out.println(myClass.getName() + " :: " + myMethodName + ":  " + string + " " + string1 + " " + string2);
    }

    public void visitLdcInsn(Object object) {
    }

    public void visitMultiANewArrayInsn(String string, int i) {
        //System.out.println(myClass.getName() + " : " + myMethodName + " : " + string);
    }

    public void visitTryCatchBlock(Label label, Label label1, Label label2, String string) {
    }

    public void visitEnd() {
    }
}
