package ru.amse.javadependencies.zhukova.data.dependencies.impl;

import ru.amse.javadependencies.zhukova.data.IJavaElement;
import ru.amse.javadependencies.zhukova.data.IClass;
import ru.amse.javadependencies.zhukova.data.dependencies.INotLeafDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.IDependency;

import java.util.*;

/**
 * @author Anna R. Zhukova
 */
public abstract class NotLeafDependency<S extends IJavaElement, D extends IDependency<IClass>> extends Dependency<S> implements INotLeafDependency<S, D> {
    private final Set<D> mySubdependencies;

    public NotLeafDependency(S server, Set<D> subdependencies) {
        super(server);
        mySubdependencies = subdependencies;
    }

    public NotLeafDependency(S server) {
        this(server, new HashSet<D>());
    }

    public Set<D> getSubdependencies() {
        return mySubdependencies;
    }

    public boolean addSubdependency(D subDependency) {
        return mySubdependencies.add(subDependency);
    }

    //todo
    public void updateSubdependency(D subDependency) {
        for (Iterator<D> i = mySubdependencies.iterator(); i.hasNext();) {
            if (i.next().getServer().getName().equals(subDependency.getServer().getName())) {
                i.remove();
                break;
            }
        }
        addSubdependency(subDependency);
    }

    public String toString(String subDependencyName, int spacesNumber) {
        StringBuilder spaces = new StringBuilder("");
        for (int j = 0; j < spacesNumber; j++) {
            spaces.append(" ");
        }
        StringBuilder result = new StringBuilder(getServer().getName());
        result.append("\n    ");
        result.append(subDependencyName);
        int i = 0;
        for (D sd : getSubdependencies()) {
            result.append(spaces.toString());
            result.append(++i);
            result.append(". ");
            result.append(sd.toString());
            result.append("\n");
        }
        return result.toString();
    }
}
