package ru.amse.javadependencies.zhukova.model.impl;

import ru.amse.javadependencies.zhukova.model.IArc;
import ru.amse.javadependencies.zhukova.model.IArcFilter;

import java.awt.*;
import static java.lang.Math.*;
import java.util.List;

/**
 * @author Anna R. Zhukova
 */
public class Arc implements IArc {
    private static IArcFilter ourFilter;

    public static void setFilter(IArcFilter filter) {
        ourFilter = filter;
    }

    public static IArcFilter getFilter() {
        return ourFilter;
    }

    private final SimpleVertex myFrom;
    private final SimpleVertex myTo;
    private final List<String> myToolTipText;

    public Arc(SimpleVertex from, SimpleVertex to, List<String> tooltipText) {
        myFrom = from;
        myTo = to;
        myToolTipText = tooltipText;
    }

    public SimpleVertex getFromVertex() {
        return myFrom;
    }

    public SimpleVertex getToVertex() {
        return myTo;
    }


    public List<String> getToolTipText() {
        return myToolTipText;
    }

    public String getToolTipHTML() {
        StringBuilder result = new StringBuilder("<html>");
        for (String s : myToolTipText) {
            if (ourFilter == null || ourFilter.accept(s)) {
                result.append(s).append("<br>");
            }
        }
        return result.toString();
    }

    public void paintIfNeeded(Graphics g) {
        if (!isHidden()) {
            paint(g);
        }
    }

    public void paint(Graphics g) {
        g.setColor(Color.BLACK);
        Point loc = myFrom.getAbsoluteLocation();
        int x1 = loc.x + myFrom.getWidth() / 2;
        int y1 = loc.y + myFrom.getHeight() / 2;
        loc = myTo.getAbsoluteLocation();
        int x2 = loc.x + myTo.getWidth() / 2;
        int y2 = loc.y + myTo.getHeight() / 2;
        g.drawLine(x1, y1, x2, y2);
        paintArrow(atan2(y2 - y1, x2 - x1), x1 + (x2 - x1) / 2, y1 + (y2 - y1) / 2, PI / 8, 10, g);
    }

    private static void paintArrow(double direction, int x0, int y0, double beta, int arrowLength, Graphics g) {
        g.setColor(Color.BLACK);
        int xr = x0 + (int) round((arrowLength * cos(PI + direction - beta)));
        int yr = y0 + (int) round((arrowLength * sin(PI + direction - beta)));
        int xl = x0 + (int) round((arrowLength * cos(PI + direction + beta)));
        int yl = y0 + (int) round((arrowLength * sin(PI + direction + beta)));

        g.drawLine(x0, y0, xr, yr);
        g.drawLine(x0, y0, xl, yl);
    }

    public boolean hitTest(Point p) {
        Point fromLocation = getFromVertex().getAbsoluteLocation();
        Point toLocation = getToVertex().getAbsoluteLocation();
        int r0 = (int) (pow(pow(fromLocation.x - p.x, 2) + pow(fromLocation.y - p.y, 2), 0.5));
        double beta = atan2(fromLocation.y - p.y, p.x - fromLocation.x) - atan2(fromLocation.y - toLocation.y, toLocation.x - fromLocation.x);
        int x0 = (int) (r0 * cos(beta));
        int y0 = (int) (r0 * sin(beta));
        int r = (int) (pow(pow(fromLocation.x - toLocation.x, 2) + pow(fromLocation.y - toLocation.y, 2), 0.5));
        return x0 >= 10 && x0 <= r - 10 && y0 >= -5 && y0 <= 5;
    }

    public boolean equals(Object obj) {
        if (!(obj instanceof Arc)) {
            return false;
        }
        Arc arc = (Arc) obj;
        return this == arc || myFrom.equals(arc.myFrom) && myTo.equals(arc.myTo);
    }

    public int hashCode() {
        return myFrom.hashCode() + 7 * myTo.hashCode();
    }

    public boolean isHidden() {
        return myFrom.isHidden() || myTo.isHidden() || "<html>".equals(getToolTipHTML());
    }
}
