package ru.amse.javadependencies.zhukova.model.impl;

import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;
import ru.amse.javadependencies.zhukova.model.IVertex;

import java.awt.*;

/**
 * @author Anna R. Zhukova
 */
public abstract class Vertex implements IVertex {
    private Point myLocation;
    private final String myName;
    private int myWidth;
    private int myHeight;
    private boolean isHidden = false;
    private int myLetterHeight = 21;
    private int myLetterWidth = 9;
    private Font myFont = new Font(Font.MONOSPACED, Font.ITALIC, 15);

    public Vertex(String name) {
        myName = name;
    }

    public Point getLocation() {
        return new Point(myLocation);
    }

    public void setLocation(Point p) {
        myLocation = new Point(p);
    }

    public int getWidth() {
        return myWidth;
    }

    public void setWidth(int width) {
        myWidth = width;
    }

    public int getHeight() {
        return myHeight;
    }

    public void setHeight(int height) {
        myHeight = height;
    }


    public void paintIfNeeded(Graphics g, GraphPane graphPane, boolean isSelected) {           
        if (!isHidden()) {
            paint(g, graphPane, isSelected);
        }
    }

    public abstract void paint(Graphics g, GraphPane graphPane, boolean isSelected);

    public void setHidden(boolean need) {
        isHidden = need;
    }

    public boolean isHidden() {
        return isHidden;
    }

    public boolean hitTest(Point p) {
        return (p.x >= (getAbsoluteLocation().x))
                && (p.x <= (getAbsoluteLocation().x + getWidth()))
                && (p.y >= (getAbsoluteLocation().y))
                && (p.y <= (getAbsoluteLocation().y + getHeight()));
    }

    public String getName() {
        return myName;
    }

    public boolean isInRectangleTest(Point left, Point right) {
        int x = getAbsoluteLocation().x;
        int y = getAbsoluteLocation().y;
        int inXLeft = Math.min(left.x, right.x);
        int inYLeft = Math.min(left.y, right.y);
        int inXRight = Math.max(left.x, right.x);
        int inYRight = Math.max(left.y, right.y);
        return !((x + getWidth() < inXLeft) || (x > inXRight)
                || (y + getHeight() < inYLeft) || (y > inYRight));
    }

    public Point getMaxPoint() {
        return new Point(getAbsoluteLocation().x + getWidth(), getAbsoluteLocation().y + getHeight());
    }

    public Point getMinPoint() {
        return new Point(getAbsoluteLocation());
    }

    public void setLetterWidth(int letterWidth) {
        myLetterWidth = letterWidth;
    }

    public void setLetterHeight(int letterHeight) {
        myLetterHeight = letterHeight;
    }

    public int getLetterWidth() {
        return myLetterWidth;
    }

    public int getLetterHeight() {
        return myLetterHeight;
    }

    public void setFont(Font font) {
        myFont = font;
    }

    public Font getFont() {
        return myFont;
    }
}
