package ru.amse.javadependencies.zhukova.modelbuilding.tree;

import ru.amse.javadependencies.zhukova.ui.tree.CheckNode;
import ru.amse.javadependencies.zhukova.ui.tree.CheckRenderer;
import ru.amse.javadependencies.zhukova.modelbuilding.tree.popupmenu.PopUpMenu;
import ru.amse.javadependencies.zhukova.model.impl.SuperVertex;
import ru.amse.javadependencies.zhukova.model.impl.SimpleVertex;
import ru.amse.javadependencies.zhukova.model.IGraphModel;

import javax.swing.*;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import java.util.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * @author Anna R. Zhukova
 */
public class TreeBuilder implements ITreeBuilder {
    private final IGraphModel myGraphModel;

    public TreeBuilder(IGraphModel graphModel) {
        myGraphModel = graphModel;
    }

    public JTree buildTree() {
        CheckNode root = new CheckNode("Packages");
        Map<String, SuperVertex> packageNameToVertex = myGraphModel.getNameToSuperVertexMap();
        List<String> packageNames = new ArrayList<String>(packageNameToVertex.keySet());
        Collections.sort(packageNames, new Comparator<String>() {

            public int compare(String o1, String o2) {
                if ("<default>".equals(o1)) {
                    return -1;
                }
                if ("<default>".equals(o2)) {
                    return 1;
                }
                return o1.compareTo(o2);
            }
        });
        for (String name : packageNames) {
            CheckNode packageNode = new CheckNode(name);
            List<SimpleVertex> classVertices = new ArrayList<SimpleVertex>(packageNameToVertex.get(name).getSimpleVertices());
            Collections.sort(classVertices, new Comparator<SimpleVertex>() {

                public int compare(SimpleVertex o1, SimpleVertex o2) {
                    String name1 = o1.getName();
                    String name2 = o2.getName();
                    return name1.compareTo(name2);
                }
            });
            for (SimpleVertex clazz : classVertices) {
                CheckNode classNode = new CheckNode(clazz.getName());
                packageNode.add(classNode);
            }
            root.add(packageNode);
        }
        final JTree tree = new JTree(root);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setCellRenderer(new CheckRenderer());
        tree.addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                TreePath treePath = tree.getPathForLocation(e.getX(), e.getY());
                PopUpMenu.setRow(tree.getRowForLocation(e.getX(), e.getY()));
                if (e.getButton() == MouseEvent.BUTTON1) {
                    if (treePath != null) {
                        CheckNode node = (CheckNode) treePath.getLastPathComponent();
                        boolean leaf = node.isLeaf();
                        if (!node.isSelected()) {
                            if (leaf) {
                                PopUpMenu.getLeafShowInstance().show(e.getComponent(),
                                        e.getX(), e.getY());
                            } else {
                                PopUpMenu.getNotLeafShowInstance().show(e.getComponent(),
                                        e.getX(), e.getY());
                            }
                        } else {
                            if (leaf) {
                                PopUpMenu.getLeafHideInstance().show(e.getComponent(),
                                        e.getX(), e.getY());
                            } else {
                                PopUpMenu.getNotLeafHideInstance().show(e.getComponent(),
                                        e.getX(), e.getY());
                            }
                        }
                    }
                }
            }
        });
        return tree;
    }
}
