package ru.amse.javadependencies.zhukova.reading;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import java.util.*;
import java.util.List;
import java.awt.*;

import ru.amse.javadependencies.zhukova.model.*;
import ru.amse.javadependencies.zhukova.model.impl.Arc;
import ru.amse.javadependencies.zhukova.model.impl.SimpleVertex;
import ru.amse.javadependencies.zhukova.model.impl.SuperVertex;
import ru.amse.javadependencies.zhukova.model.impl.ArcFilter;

/**
 * @author Anna R. Zhukova
 */
class ParserHandler extends DefaultHandler {
    private final IGraphModel myGraphModel;
    private final Map<String, SimpleVertex> myNameToSimpleVertex = new HashMap<String, SimpleVertex>();
    private final Map<String, SuperVertex> myNameToSuperVertex = new HashMap<String, SuperVertex>();
    private SimpleVertex myFromSimpleVertex;
    private SimpleVertex myToSimpleVertex;
    private List<String> myLabel;
    private SuperVertex mySuperVertex;
    private IVertex myVertex;
    private IArcFilter myArcFilter;

    public ParserHandler(IGraphModel graphModel) {
        myGraphModel = graphModel;
        myArcFilter = Arc.getFilter();
        if (myArcFilter == null) {
            myArcFilter = new ArcFilter();
            Arc.setFilter(myArcFilter);
        }
        myArcFilter.removeAllBarriers();
    }

    public void startElement(String uri, String localName,
                             String qName, Attributes attributes) throws SAXException {
        if ("superVertex".equals(qName)) {
            String name = attributes.getValue("name");
            SuperVertex v = new SuperVertex(new Point(Integer.parseInt(attributes.getValue("x")),
                    Integer.parseInt(attributes.getValue("y"))), name);
            v.setWidth(Integer.parseInt(attributes.getValue("width")));
            v.setHeight(Integer.parseInt(attributes.getValue("height")));
            v.setHidden(Boolean.parseBoolean(attributes.getValue("isHidden")));
            v.setLetterWidth(Integer.parseInt(attributes.getValue("letterWidth")));
            v.setLetterHeight(Integer.parseInt(attributes.getValue("letterHeight")));
            myNameToSuperVertex.put(name, v);
            myGraphModel.addVertex(v);
            mySuperVertex = v;
            myVertex = v;
        } else if ("simpleVertex".equals(qName)) {
            String name = attributes.getValue("name");
            SimpleVertex v = new SimpleVertex(new Point(Integer.parseInt(attributes.getValue("x")),
                    Integer.parseInt(attributes.getValue("y"))), name, mySuperVertex);
            v.setWidth(Integer.parseInt(attributes.getValue("width")));
            v.setHeight(Integer.parseInt(attributes.getValue("height")));
            v.setHidden(Boolean.parseBoolean(attributes.getValue("isHidden")));
            v.setLetterWidth(Integer.parseInt(attributes.getValue("letterWidth")));
            v.setLetterHeight(Integer.parseInt(attributes.getValue("letterHeight")));
            myNameToSimpleVertex.put(mySuperVertex.getName() + "." + name, v);
            myGraphModel.addVertex(v);
            mySuperVertex.addSimpleVertex(v);
            myVertex = v;
        } else if ("from".equals(qName)) {
            myFromSimpleVertex = myNameToSimpleVertex.get(attributes.getValue("surname") + "." + attributes.getValue("name"));
        } else if ("to".equals(qName)) {
            myToSimpleVertex = myNameToSimpleVertex.get(attributes.getValue("surname") + "." + attributes.getValue("name"));
        } else if ("Label".equals(qName)) {
            myLabel = new ArrayList<String>();
        } else if ("Entry".equals(qName)) {
            //Arc.myTypeToShow.put(attributes.getValue("key"), Boolean.valueOf(attributes.getValue("value")));
        } else if ("Font".equals(qName)) {
            myVertex.setFont(new Font(attributes.getValue("name"), Integer.valueOf(attributes.getValue("style")), Integer.valueOf(attributes.getValue("size"))));
        } else if ("barrier".equals(qName)) {
            myArcFilter.addBarrier(attributes.getValue("name"));
        } else if ("pr".equals(qName)) {
            myLabel.add(attributes.getValue("name"));
        }
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        if ("ar".equals(qName)) {
            Arc arc = new Arc(myFromSimpleVertex, myToSimpleVertex, myLabel);
            myGraphModel.addArc(arc);
        }
    }

    public void endDocument ()	throws SAXException {
        myGraphModel.setNameToSimpleVertexMap(myNameToSimpleVertex);
        myGraphModel.setNameToSuperVertexMap(myNameToSuperVertex);
    }
}
