package ru.amse.javadependencies.zhukova.ui;

import ru.amse.javadependencies.zhukova.reading.DependencyGraphReader;
import ru.amse.javadependencies.zhukova.saving.DependencyGraphSaver;
import ru.amse.javadependencies.zhukova.modelbuilding.tree.TreeBuilder;
import ru.amse.javadependencies.zhukova.model.impl.GraphModel;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.io.File;

/**
 * @author Anna R. Zhukova
 */
public class FileChoosers {
    private static Main ourMain;

    private static final JFileChooser ourFileChooser = new JFileChooser();

    private static final FileFilter ourFilter = new FileFilter() {
        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }
            String path = f.getAbsolutePath();
            return path.endsWith(".jar") || path.endsWith(".zip") || path.endsWith(".class");
        }

        public String getDescription() {
            return null;
        }
    };

    public static void setMain(Main main) {
        ourMain = main;
    }

    public static void getOpenSavedFileChooser() {
        ourFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        ourFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        ourFileChooser.removeChoosableFileFilter(ourFilter);
        int value = ourFileChooser.showDialog(ourMain.getFrame(), "Select File");
        if (value == JFileChooser.APPROVE_OPTION) {
            try {
                ourMain.getGraphPane().deselectVertices();
                GraphModel gm = DependencyGraphReader.read(ourMain.getGraphPane(), ourFileChooser.getSelectedFile());
                ourMain.setGraphModel(gm);
                ourMain.setTree(new TreeBuilder(gm).buildTree());
                ourMain.updateDependencyTree();
                ourMain.setChanged(true);
            } catch (Exception exc) {
                exc.printStackTrace();
                JOptionPane.showMessageDialog(ourFileChooser, "Cannot open this file");
            }
        }
    }

    public static File getOpenFileChooser() {
        ourFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        ourFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        ourFileChooser.addChoosableFileFilter(ourFilter);
        int value = ourFileChooser.showDialog(ourMain.getFrame(), "Select File");
        if (value == JFileChooser.APPROVE_OPTION) {
            return ourFileChooser.getSelectedFile();
        }
        return null;
    }

    public static void getSaveFileChooser(boolean exitAfterSaving) {
        ourFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        ourFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        ourFileChooser.removeChoosableFileFilter(ourFilter);
        int value = ourFileChooser.showDialog(ourMain.getFrame(), "Select File");
        if (value == JFileChooser.APPROVE_OPTION) {
            File f = ourFileChooser.getSelectedFile();
            if (f != null) {
                try {
                    if (f.isFile()) {
                        int replace = JOptionPane.showConfirmDialog(ourFileChooser, "File " + f.getName() + " already exists. Replace it?");
                        if (replace == JOptionPane.OK_OPTION) {
                            DependencyGraphSaver.save(ourMain.getGraphModel(), f);
                        }
                    } else {
                        f.createNewFile();
                        DependencyGraphSaver.save(ourMain.getGraphModel(), f);
                    }
                    if (exitAfterSaving) {
                        System.exit(0);
                    }
                } catch (Exception e) {
                    JOptionPane.showMessageDialog(ourFileChooser, "Cannot save");
                }
            }
        }
    }
}
