package ru.amse.javadependencies.zhukova.ui.graphpane.tools;

import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;
import ru.amse.javadependencies.zhukova.ui.graphpane.IGraphPane;
import ru.amse.javadependencies.zhukova.model.impl.SimpleVertex;
import ru.amse.javadependencies.zhukova.model.impl.SuperVertex;
import ru.amse.javadependencies.zhukova.model.IVertex;

import java.awt.event.MouseEvent;
import java.awt.*;
import java.util.Set;

/**
 * @author Anna R. Zhukova
 */
public class SelectingTool extends Tool {
    private static boolean ourIsOnlySuperVerticesSelection;
    private static Point ourLeft;
    private static int ourRectangleX;
    private static int ourRectangleY;
    private static int ourRectangleWidth;
    private static int ourRectangleHeight;
    private static final SelectingTool ourInstance = new SelectingTool();

    private SelectingTool() {
        super();
    }

    public static boolean isOnlySuperVerticesSelection() {
        return ourIsOnlySuperVerticesSelection;
    }

    public static void setOnlySuperVerticesSelection(boolean b) {
        ourIsOnlySuperVerticesSelection = b;
    }

    public void mousePressed(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1) {
            IGraphPane graphPane = Tool.getGraphPane();
            Tool.getUndoManager().update();
            Point mouseLocation = e.getPoint();
            Set<IVertex> vertices = graphPane.getSelectedVertices();
            for (IVertex v : vertices) {
                if (v.hitTest(mouseLocation)) {
                    moveOn(mouseLocation);
                    return;
                }
            }
            graphPane.deselectVertices();
            if (!ourIsOnlySuperVerticesSelection) {
                Set<SimpleVertex> simpleVertices = graphPane.getGraphModel().getSimpleVertices();
                for (SimpleVertex v : simpleVertices) {
                    if (v.hitTest(mouseLocation)) {
                        graphPane.selectVertex(v);
                        moveOn(mouseLocation);
                        return;
                    }
                }
            }
            Set<SuperVertex> superVertices = graphPane.getGraphModel().getSuperVertices();
            for (SuperVertex v : superVertices) {
                boolean leftX = v.leftBorderTest(mouseLocation);
                boolean rightX = v.rightBorderTest(mouseLocation);
                boolean topY = v.topBorderTest(mouseLocation);
                boolean bottomY = v.bottomBorderTest(mouseLocation);
                if (leftX || rightX || topY || bottomY) {
                    resizeOn(mouseLocation, v, leftX, rightX, topY, bottomY);
                } else if (v.hitTest(mouseLocation)) {
                    graphPane.selectVertex(v);
                    moveOn(mouseLocation);
                    return;
                }
            }
            ourLeft = mouseLocation;
        }
    }

    private void resizeOn(Point mouseLocation, SuperVertex v,
                          boolean left, boolean right, boolean top, boolean bottom) {
        Tool.getGraphPane().setTool(ResizingTool.getInstance(mouseLocation, v,
                left, right, top, bottom));
    }

    private void moveOn(Point mouseLocation) {
        Tool.getGraphPane().setTool(MovingTool.getInstance(mouseLocation));
    }

    public void mouseReleased(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1) {
            Point mouseLocation = e.getPoint();
            GraphPane graphPane = Tool.getGraphPane();
            if (mouseLocation.equals(ourLeft)) {
                if (ourIsOnlySuperVerticesSelection) {
                    Set<SuperVertex> vertices = graphPane.getGraphModel().getSuperVertices();
                    for (IVertex v : vertices) {
                        if (v.hitTest(mouseLocation)) {
                            graphPane.selectVertex(v);
                            break;
                        }
                    }
                } else {
                    Set<IVertex> vertices = graphPane.getGraphModel().getVertices();
                    for (IVertex v : vertices) {
                        if (v.hitTest(mouseLocation)) {
                            graphPane.selectVertex(v);
                            break;
                        }
                    }
                }
            } else {
                Set<SuperVertex> superVertices = graphPane.getGraphModel().getSuperVertices();
                for (SuperVertex v : superVertices) {
                    if (v.isInRectangleTest(ourLeft, mouseLocation)) {
                        graphPane.selectVertex(v);
                    }
                }
                if (!ourIsOnlySuperVerticesSelection) {
                    Set<SimpleVertex> simpleVertices = graphPane.getGraphModel().getSimpleVertices();
                    for (SimpleVertex v : simpleVertices) {
                        if (v.isInRectangleTest(ourLeft, mouseLocation) && !graphPane.getSelectedVertices().contains(v.getSuperVertex())) {
                            graphPane.selectVertex(v);
                        }
                    }
                }
            }
            graphPane.repaint();
        }
        ourRectangleWidth = 0;
        ourRectangleHeight = 0;
    }

    public void mouseDragged(MouseEvent e) {
        Point p = e.getPoint();
        ourRectangleX = Math.min(ourLeft.x, p.x);
        ourRectangleY = Math.min(ourLeft.y, p.y);
        ourRectangleWidth = Math.abs(ourLeft.x - p.x);
        ourRectangleHeight = Math.abs(ourLeft.y - p.y);
        Tool.getGraphPane().repaint();
    }

    public void paint(Graphics g) {
        g.setColor(Color.LIGHT_GRAY);
        g.drawRect(ourRectangleX, ourRectangleY, ourRectangleWidth, ourRectangleHeight);
    }

    public static SelectingTool getInstance() {
        return ourInstance;
    }
}
